#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 ~ 2012 Deepin, Inc.
#               2011 ~ 2012 Wang Yong
# 
# Author:     Wang Yong <lazycat.manatee@gmail.com>
# Maintainer: Wang Yong <lazycat.manatee@gmail.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Import skin and theme, those must before at any other modules.
# from skin import ui_theme, app_theme
from dtk.ui.skin_config import skin_config
from dtk.ui.theme import Theme, ui_theme
from dtk.ui.utils import get_parent_dir
import os
import ConfigParser
import gobject
import threading

# Init skin config.
skin_config.init_skin(
    "blue",
    os.path.join(get_parent_dir(__file__, 2), "skin"),
    os.path.expanduser("~/.config/deepin-hardware-detect/skin"),
    os.path.expanduser("~/.config/deepin-hardware-detect/skin_config.ini"),
    "deepin-hardware-detect",
    "1.0"
    )

# Create application theme.
app_theme = Theme(
    os.path.join(get_parent_dir(__file__, 2), "app_theme"),
    os.path.expanduser("~/.config/deepin-hardware-detect/theme")
    )

# Set theme.
skin_config.load_themes(ui_theme, app_theme)

from dtk.ui.application import Application
from dtk.ui.label import Label
from dtk.ui.button import Button
from dtk.ui.combo import ComboBox
from dtk.ui.utils import container_remove_all
import os
import gtk
import apt
import apt.debfile
import apt_pkg

current_dir = os.path.dirname(__file__)

def postGUI(func):
    def wrap(*a, **kw):
        gtk.gdk.threads_enter()
        ret = func(*a, **kw)
        gtk.gdk.threads_leave()
        return ret
    return wrap

@postGUI
def show_message(label, message):
    label.set_text(message)

class InstallThread(threading.Thread):
    def __init__(self, i_list, finish_callback):
        threading.Thread.__init__(self)
        self.i_list = i_list
        self.finish_callback = finish_callback
        self.busy = False
        apt_pkg.init()

    def run(self):
        while len(self.i_list) > 0:
            if self.busy == False:
                self.busy = True
                task = self.i_list.pop(0)
                show_message(task[0], "正在安装%s驱动 ..." % (task[1]))
                pkg = apt.debfile.DebPackage(os.path.join(current_dir, task[2]), apt.Cache())
                os.spawnlp(os.P_WAIT, "dpkg", "dpkg", "-i", os.path.join(current_dir, pkg.filename))
                show_message(task[0], "%s驱动安装完成" % (task[1]))
                self.busy = False
                
        self.finish_callback()        

def i_driver(arg, parse_hardware, install_button, 
             graphics_card, g_status_label, sound_card, s_status_label, net_card, n_status_label,
             finish_callback):
    install_button.set_label("正在安装驱动")
    install_button.set_sensitive(False)
    
    parser = ConfigParser.ConfigParser()
    parser.read(os.path.join(current_dir, "config.ini"))
    i_list = []

    if parse_hardware:
        for item in graphics_card:
            try:
                driver_name = parser.get("显卡", item)
                i_list.append([g_status_label, "显卡", driver_name])
            except Exception:
                continue
        
        for item in sound_card:
            try:
                driver_name = parser.get("声卡", item)
                i_list.append([s_status_label, "声卡", driver_name])
            except Exception:
                continue
        
        for item in net_card:
            try:
                driver_name = parser.get("网卡", item)
                i_list.append([n_status_label, "网卡", driver_name])
            except Exception:
                continue
    else:
        i_list.append([g_status_label, "显卡", "deepin-meta-graphics-driver_0.1_all.deb"])
        i_list.append([s_status_label, "声卡", "deepin-meta-sound-driver_0.1_all.deb"])
        i_list.append([n_status_label, "网卡", "deepin-meta-ethernet-driver_0.1_all.deb"])

    i = InstallThread(i_list, finish_callback)
    i.start()
    
def change_button(button_box, finish_button):
    '''Change button when finish.'''
    container_remove_all(button_box)
    button_box.add(finish_button)
    
    finish_button.connect("clicked", lambda w: gtk.main_quit())
    
    # window.show_all()
    button_box.show_all()

if __name__ == "__main__":
    parse_hardware = True
    if len(os.sys.argv) >= 2 and os.sys.argv[1] == "--test":
        parse_hardware = False
    
    # Init application.
    application = Application()
    application.set_skin_preview(app_theme.get_pixbuf("frame.png"))
    application.add_titlebar(
        ["theme", "max", "min", "close"],
        None,
        "深度硬件探测工具"
        )
    
    # Add device list.
    graphics_cards = filter(lambda a: a != '', os.popen("lspci | grep VGA | cut -d : -f 3 | sed 's/^ //g'").read().split('\n'))
    sound_cards = filter(lambda a: a != '', os.popen("lspci | grep \"Audio device\" | cut -d : -f 3 | sed 's/^ //g'").read().split('\n'))
    net_cards = filter(lambda a: a != '', os.popen("lspci | grep Ethernet | cut -d : -f 3 | sed 's/^ //g'").read().split('\n')) + filter(lambda a: a != '', os.popen("lspci | grep Network | cut -d : -f 3 | sed 's/^ //g'").read().split('\n'))
    
    # Add device widgets.
    device_table = gtk.Table(6, 2)
    device_table.set_row_spacings(10)
    device_table.set_col_spacings(10)
    
    graphics_card_label = Label("显卡", enable_gaussian=True, text_size=12)
    graphics_card_combobox = ComboBox(map(lambda i: (i, i), graphics_cards))
    graphics_card_combobox_hbox = gtk.HBox()
    graphics_card_combobox_hbox.pack_start(graphics_card_label, False, False, 5)
    graphics_card_combobox_hbox.pack_start(graphics_card_combobox, False, True, 5)
    graphics_card_info_label = Label("未探测到匹配的显卡驱动", text_size=10)
    
    sound_card_label = Label("声卡", enable_gaussian=True, text_size=12)
    sound_card_combobox = ComboBox(map(lambda i: (i, i), sound_cards))
    sound_card_combobox_hbox = gtk.HBox()
    sound_card_combobox_hbox.pack_start(sound_card_label, False, False, 5)
    sound_card_combobox_hbox.pack_start(sound_card_combobox, False, True, 5)
    sound_card_info_label = Label("未探测到匹配的声卡驱动", text_size=10)
    
    net_card_label = Label("网卡", enable_gaussian=True, text_size=12)
    net_card_combobox = ComboBox(map(lambda i: (i, i), net_cards))
    net_card_combobox_hbox = gtk.HBox()
    net_card_combobox_hbox.pack_start(net_card_label, False, False, 5)
    net_card_combobox_hbox.pack_start(net_card_combobox, False, True, 5)
    net_card_info_label = Label("未探测到匹配的网卡驱动", text_size=10)

    # Install button.
    install_button = Button("一键安装驱动")
    finish_button = Button("完成安装")
    button_box = gtk.HBox()
    button_box.pack_start(install_button, False, False)
    button_box_align = gtk.Alignment()
    button_box_align.set(1, 0.5, 0, 0)
    button_box_align.set_padding(5, 5, 5, 5)
    button_box_align.add(button_box)
    install_button.connect(
        "clicked",
        i_driver,
        parse_hardware,
        install_button,
        graphics_cards, 
        graphics_card_info_label,
        sound_cards, 
        sound_card_info_label,
        net_cards, 
        net_card_info_label,
        lambda : change_button(button_box, finish_button)
        )

    # Connect box.
    device_table.attach(graphics_card_combobox_hbox, 0, 2, 0, 1, xoptions=gtk.FILL, yoptions=gtk.SHRINK)
    device_table.attach(graphics_card_info_label, 0, 2, 1, 2, xoptions=gtk.FILL, yoptions=gtk.SHRINK, xpadding=50)
    device_table.attach(sound_card_combobox_hbox, 0, 2, 2, 3, xoptions=gtk.FILL, yoptions=gtk.SHRINK)
    device_table.attach(sound_card_info_label, 0, 2, 3, 4, xoptions=gtk.FILL, yoptions=gtk.SHRINK, xpadding=50)
    device_table.attach(net_card_combobox_hbox, 0, 2, 4, 5, xoptions=gtk.FILL, yoptions=gtk.SHRINK)
    device_table.attach(net_card_info_label, 0, 2, 5, 6, xoptions=gtk.FILL, yoptions=gtk.SHRINK, xpadding=50)
    
    vbox = gtk.VBox()
    vbox.pack_start(device_table, True, True)
    
    vbox.pack_start(button_box_align)
    
    valign = gtk.Alignment()
    valign.set(0.5, 0.5, 0, 0)
    valign.set_padding(10, 10, 10, 10)
    valign.add(vbox)
    
    application.main_box.pack_start(valign, True, True)
    
    # Init match drivers.
    parser = ConfigParser.ConfigParser()
    parser.read(os.path.join(current_dir, "config.ini"))
    if parse_hardware:
        for item in graphics_cards:
            try:
                driver_name = parser.get("显卡", item)
                graphics_card_info_label.set_text("探测到显卡驱动: %s" % (driver_name))
            except Exception:
                continue
        
        for item in sound_cards:
            try:
                driver_name = parser.get("声卡", item)
                sound_card_info_label.set_text("探测到声卡驱动: %s" % (driver_name))
            except Exception:
                continue
        
        for item in net_cards:
            try:
                driver_name = parser.get("网卡", item)
                net_card_info_label.set_text("探测到网卡驱动: %s" % (driver_name))
            except Exception:
                continue
    else:
        graphics_card_info_label.set_text("探测到显卡驱动: %s" % ("deepin-meta-graphics-driver_0.1_all.deb"))
        sound_card_info_label.set_text("探测到声卡驱动: %s" % ("deepin-meta-sound-driver_0.1_all.deb"))
        net_card_info_label.set_text("探测到网卡驱动: %s" % ("deepin-meta-ethernet-driver_0.1_all.deb"))
        
    # Run.
    gobject.threads_init()
    application.run()
